package finance.chai.chai_webview_plugin;

import android.annotation.TargetApi;
import android.app.Activity;
import android.app.AlertDialog;
import android.content.Context;
import android.content.DialogInterface;
import android.content.Intent;
import android.graphics.Bitmap;
import android.net.Uri;
import android.os.Build;
import android.view.View;
import android.webkit.CookieManager;
import android.webkit.CookieSyncManager;
import android.webkit.GeolocationPermissions;
import android.webkit.JsResult;
import android.webkit.ValueCallback;
import android.webkit.WebChromeClient;
import android.webkit.WebSettings;
import android.webkit.WebView;

import androidx.annotation.NonNull;
import androidx.annotation.Nullable;

import java.lang.ref.WeakReference;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import io.flutter.plugin.common.MethodCall;
import io.flutter.plugin.common.MethodChannel;
import io.flutter.plugin.common.PluginRegistry;
import io.flutter.plugin.platform.PlatformView;
import timber.log.Timber;

import static android.app.Activity.RESULT_OK;

/**
 *
 */
class ChaiWebView implements PlatformView, MethodChannel.MethodCallHandler, PluginRegistry.ActivityResultListener, WebViewClientListener {

    private static final String BASE_CHANNEL_NAME = "chai_webview_plugin";

    private static String getChannelName(int id) {
        return BASE_CHANNEL_NAME + "/" + id;
    }

    private static final int REQ_FILE_CHOOSER = 0x01;


    @NonNull
    private final ObservableWebView webView;
    @NonNull
    private final MethodChannel methodChannel;

    @Nullable
    private WeakReference<Activity> activityRef;

    @Nullable
    private ValueCallback<Uri> uploadMessageCallback;
    @Nullable
    private ValueCallback<Uri[]> uploadMessageArrayCallback;

    public ChaiWebView(Context context, PluginRegistry.Registrar registrar, int id, Object args) {
        registrar.addActivityResultListener(this);
        activityRef = new WeakReference<>(registrar.activity());
        webView = createWebView(context, args);
        Timber.d("create MethodChannel name %s", getChannelName(id));
        methodChannel = new MethodChannel(registrar.messenger(), getChannelName(id));
        methodChannel.setMethodCallHandler(this);
    }

    @SuppressWarnings("unchecked")
    @NonNull
    private ObservableWebView createWebView(final Context context, Object args) {
        Timber.d("createWebView");
        @Nullable WebviewBuildParams params = null;
        try {
            params = WebviewBuildParams.newInstance((Map<String, Object>) args);
        } catch (Exception e) {
            Timber.e(e, "access params fail");
        }

        ObservableWebView webView = new ObservableWebView(context);
        BrowserClient webClient = new BrowserClient(this) {
            @Override
            public void onPageFinished(WebView view, String url) {
                super.onPageFinished(view, url);
                view.setVisibility(View.VISIBLE);
            }
        };
        if (params != null) {
            setupWebviewSettings(webView, params);
            webClient.updateInvalidUrlRegex(params.invalidUrlRegex);
        }

        webView.setWebViewClient(webClient);
        webView.setWebChromeClient(new WebChromeClient() {
            private final String TAG = "WebviewClient";

            //The undocumented magic method override
            //Eclipse will swear at you if you try to put @Override here
            // For Android 3.0+
            public void openFileChooser(ValueCallback<Uri> uploadMsg) {

                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(null);
                }
                uploadMessageCallback = uploadMsg;
                Intent i = new Intent(Intent.ACTION_GET_CONTENT);
                i.addCategory(Intent.CATEGORY_OPENABLE);
                i.setType("image/*");

                @Nullable Activity activity = getActivity(activityRef);
                if (activity != null) {
                    activity.startActivityForResult(Intent.createChooser(i, "File Chooser"), REQ_FILE_CHOOSER);
                }

            }

            // For Android 3.0+
            public void openFileChooser(ValueCallback uploadMsg, String acceptType) {
                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(null);
                }
                uploadMessageCallback = uploadMsg;
                Intent i = new Intent(Intent.ACTION_GET_CONTENT);
                i.addCategory(Intent.CATEGORY_OPENABLE);
                i.setType("*/*");
                @Nullable Activity activity = getActivity(activityRef);
                if (activity != null) {
                    activity.startActivityForResult(
                            Intent.createChooser(i, "File Browser"),
                            REQ_FILE_CHOOSER);
                }
            }

            //For Android 4.1
            public void openFileChooser(ValueCallback<Uri> uploadMsg, String acceptType, String capture) {
                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(null);
                }
                uploadMessageCallback = uploadMsg;
                Intent i = new Intent(Intent.ACTION_GET_CONTENT);
                i.addCategory(Intent.CATEGORY_OPENABLE);
                i.setType("image/*");
                @Nullable Activity activity = getActivity(activityRef);
                if (activity != null) {
                    activity.startActivityForResult(Intent.createChooser(i, "File Chooser"), REQ_FILE_CHOOSER);
                }

            }

            //For Android 5.0+
            public boolean onShowFileChooser(
                    WebView webView, ValueCallback<Uri[]> filePathCallback,
                    FileChooserParams fileChooserParams) {

                Timber.d("onShowFileChooser");
                if (uploadMessageArrayCallback != null) {
                    uploadMessageArrayCallback.onReceiveValue(null);
                }
                uploadMessageArrayCallback = filePathCallback;

                Intent contentSelectionIntent = new Intent(Intent.ACTION_GET_CONTENT);
                contentSelectionIntent.addCategory(Intent.CATEGORY_OPENABLE);
                contentSelectionIntent.setType("*/*");
                Intent[] intentArray;
                intentArray = new Intent[0];

                Intent chooserIntent = new Intent(Intent.ACTION_CHOOSER);
                chooserIntent.putExtra(Intent.EXTRA_INTENT, contentSelectionIntent);
                chooserIntent.putExtra(Intent.EXTRA_TITLE, "Image Chooser");
                chooserIntent.putExtra(Intent.EXTRA_INITIAL_INTENTS, intentArray);

                @Nullable Activity activity = getActivity(activityRef);
                if (activity != null) {
                    activity.startActivityForResult(chooserIntent, REQ_FILE_CHOOSER);
                }

                return true;
            }

            public boolean onJsConfirm(WebView view, String url, String message, final JsResult result) {

                new AlertDialog.Builder(view.getContext())
                        .setMessage(message)
                        .setPositiveButton(android.R.string.ok,
                                new DialogInterface.OnClickListener() {
                                    public void onClick(DialogInterface dialog, int which) {
                                        result.confirm();
                                    }
                                })
                        .setNegativeButton(android.R.string.cancel,
                                new DialogInterface.OnClickListener() {
                                    @Override
                                    public void onClick(DialogInterface dialog, int which) {
                                        result.cancel();
                                    }
                                })
                        .setCancelable(false)
                        .create()
                        .show();
                return true;
            }
        });

        if (params != null && params.url != null) {
            Timber.d("load url : " + params.url);
            if (params.headers != null) {
                webView.loadUrl(params.url, params.headers);
            } else {
                webView.loadUrl(params.url);
            }
        }
        return webView;
    }

    @Override
    public View getView() {
        return webView;
    }

    @Override
    public void dispose() {

    }


    @Override
    public void onMethodCall(MethodCall methodCall, @NonNull MethodChannel.Result result) {
        switch (methodCall.method) {
            case "eval":
                eval(methodCall, result);
                break;
            case "resetAuthCookies":
                resetAuthCookies(methodCall);
                result.success(null);
                break;
            case "reloadUrl":
                reloadUrl(methodCall);
                result.success(null);
                break;
            case "show":
                show();
                result.success(null);
                break;
            case "hide":
                hide();
                result.success(null);
                break;
            case "stopLoading":
                stopLoading();
                result.success(null);
                break;
            case "cleanCookies":
                cleanCookies();
                result.success(null);
                break;
            case "back":
                back();
                result.success(null);
                break;
            case "forward":
                forward();
                break;
            case "reload":
                reload();
                break;
            default:
                result.notImplemented();
        }
    }

    private void hide() {
        webView.setVisibility(View.INVISIBLE);
    }

    private void show() {
        webView.setVisibility(View.VISIBLE);
    }

    private void reloadUrl(MethodCall methodCall) {
        String url = methodCall.argument("url");
        webView.loadUrl(url);
    }


    private void resetAuthCookies(MethodCall call) {
        Map<String, String> authCookies = call.argument("authCookies");
        setAuthCookies(authCookies);
    }

    @TargetApi(Build.VERSION_CODES.KITKAT)
    private void eval(MethodCall call, final MethodChannel.Result result) {
        String code = call.argument("code");

        webView.evaluateJavascript(code, new ValueCallback<String>() {
            @Override
            public void onReceiveValue(String value) {
                result.success(value);
            }
        });
    }

    private void reload() {
        webView.reload();
    }

    private void forward() {
        if (webView.canGoForward()) {
            webView.goForward();
        }
    }

    private void back() {
        if (webView.canGoBack()) {
            webView.goBack();
        }
    }

    private void cleanCookies() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            CookieManager.getInstance().removeAllCookies(new ValueCallback<Boolean>() {
                @Override
                public void onReceiveValue(Boolean aBoolean) {

                }
            });
        } else {
            CookieManager.getInstance().removeAllCookie();
        }

    }


    private void stopLoading() {
        webView.stopLoading();
    }


    private void setAuthCookies(Map<String, String> authCookies) {
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            CookieSyncManager.createInstance(webView.getContext());
        }
        CookieManager cookieManager = CookieManager.getInstance();
        cookieManager.setAcceptCookie(true);

        for (String key : authCookies.keySet()) {
            String value = authCookies.get(key);
            cookieManager.setCookie(".chai.finance", key + "=" + value);
        }
        if (Build.VERSION.SDK_INT < Build.VERSION_CODES.LOLLIPOP) {
            CookieSyncManager.getInstance().sync();
        }
    }

    private void setupWebviewSettings(WebView webView, @NonNull WebviewBuildParams params) {
        WebSettings settings = webView.getSettings();
        settings.setJavaScriptEnabled(params.withJavascript);
        settings.setBuiltInZoomControls(params.withZoom);
        settings.setSupportZoom(params.withZoom);
        settings.setDomStorageEnabled(params.withLocalStorage);
        settings.setJavaScriptCanOpenWindowsAutomatically(params.supportMultipleWindows);
        settings.setSupportMultipleWindows(params.supportMultipleWindows);
        settings.setAppCacheEnabled(params.appCacheEnabled);
        settings.setAllowFileAccessFromFileURLs(params.allowFileURLs);
        settings.setAllowUniversalAccessFromFileURLs(params.allowFileURLs);
        settings.setUseWideViewPort(params.useWideViewPort);


        if (params.geolocationEnabled) {
            settings.setGeolocationEnabled(true);
            webView.setWebChromeClient(new WebChromeClient() {
                @Override
                public void onGeolocationPermissionsShowPrompt(String origin, GeolocationPermissions.Callback callback) {
                    callback.invoke(origin, true, false);
                }
            });
        }

        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
            settings.setMixedContentMode(WebSettings.MIXED_CONTENT_COMPATIBILITY_MODE);
        }

        if (params.clearCache) {
            webView.clearCache(true);
            webView.clearFormData();
        }

        if (params.hidden) {
            webView.setVisibility(View.GONE);
        }

        if (params.clearCookies) {
            cleanCookies();
        }

        if (params.userAgent != null) {
            settings.setUserAgentString(params.userAgent);
        }

        if (!params.scrollBar) {
            webView.setVerticalScrollBarEnabled(false);
        }

        if (params.authCookies != null) {
            setAuthCookies(params.authCookies);
        }
    }

    @Nullable
    private Activity getActivity(@Nullable WeakReference<Activity> activityRef) {
        if (activityRef == null) {
            return null;
        }
        return activityRef.get();
    }

    /// WebviewClientListener ///

    @Override
    public void onPageStarted(String url, Bitmap favicon) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", url);
        data.put("type", "startLoad");
        sendMessage("onState", data);
    }

    @Override
    public void onPageFinished(String url) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", url);
        sendMessage("onUrlChanged", data);

        data.put("type", "finishLoad");
        sendMessage("onState", data);
    }

    @Override
    public void onShouldOverrideUrlLoading(String url, boolean invalid) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", url);
        data.put("type", invalid ? "abortLoad" : "shouldStart");
        sendMessage("onState", data);
    }

    @Override
    public void onReceivedError(int errorCode, String description, String failingUrl) {
        Map<String, Object> data = new HashMap<>();
        data.put("url", failingUrl);
        data.put("code", errorCode);
        sendMessage("onHttpError", data);
    }

    private void sendMessage(String method, Map<String, Object> data) {
        StringBuilder dataStringBuilder = new StringBuilder();
        dataStringBuilder.append("{");
        Iterator<Map.Entry<String, Object>> iterator = data.entrySet().iterator();
        while (iterator.hasNext()) {
            Map.Entry<String, Object> entry = iterator.next();
            dataStringBuilder.append(entry.getKey()).append(" : ").append(entry.getValue().toString());
            if (iterator.hasNext()) {
                dataStringBuilder.append(", ");
            }
        }
        dataStringBuilder.append("}");

        Timber.d("send Message to \"" + method + "\"" + " data " + dataStringBuilder.toString());
        methodChannel.invokeMethod(method, data);
    }

    /// PluginRegistry.ActivityResultListener ///
    @Override
    public boolean onActivityResult(int requestCode, int resultCode, Intent data) {
        if (requestCode == REQ_FILE_CHOOSER) {
            if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.LOLLIPOP) {
                Uri[] results = null;
                if (resultCode == Activity.RESULT_OK && data != null) {
                    String dataString = data.getDataString();
                    if (dataString != null) {
                        results = new Uri[]{Uri.parse(dataString)};
                    }
                }
                if (uploadMessageArrayCallback != null) {
                    uploadMessageArrayCallback.onReceiveValue(results);
                    uploadMessageArrayCallback = null;
                }
            } else {
                Uri result = null;
                if (resultCode == RESULT_OK && data != null) {
                    result = data.getData();
                }
                if (uploadMessageCallback != null) {
                    uploadMessageCallback.onReceiveValue(result);
                    uploadMessageCallback = null;
                }
            }
            return true;
        }
        return false;
    }
}
