//
//  ChaiWebPlatformView.m
//  chai_bio_auth
//
//  Created by Jeff Woo on 04/09/2019.
//

#import "ChaiWebPlatformView.h"

static NSString *const CHANNEL_NAME = @"chai_webview_plugin";

@implementation ChaiWebPlatformView
- (id)initWithArgs:(id)args
    viewIdentifier:(int64_t)viewId
   binaryMessenger: (NSObject<FlutterBinaryMessenger>*)messenger
             frame:(CGRect)rect
    viewController:(UIViewController *)_viewController {
  self = [super init];
  if (self) {
    NSString *channelName = [NSString stringWithFormat:@"%@/%lld", CHANNEL_NAME, viewId];
    channel = [FlutterMethodChannel methodChannelWithName:channelName
                                          binaryMessenger:messenger];
    
    viewController = _viewController;
    FlutterMethodCall *call = [FlutterMethodCall methodCallWithMethodName:@"none" arguments:args];
    [self initWebview:call];
    [self initMethodHandler];
    self.webview.frame = rect;
    NSLog(@"webview making %lld", viewId);
  }
  return self;
}

- (void)dealloc {
  NSLog(@"webview deallocating!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!");
  [self.webview stopLoading];
  [self.webview removeObserver:self forKeyPath:@"canGoBack"];
}

- (void)initWebview:(FlutterMethodCall*)call {
  NSNumber *clearCache = call.arguments[@"clearCache"];
  NSNumber *clearCookies = call.arguments[@"clearCookies"];
  _enableAppScheme = call.arguments[@"enableAppScheme"];
  NSString *userAgent = call.arguments[@"userAgent"];
  _invalidUrlRegex = call.arguments[@"invalidUrlRegex"];
  NSDictionary *authCookies = call.arguments[@"authCookies"];
  NSNumber *hidden = call.arguments[@"hidden"];
  NSNumber *withZoom = call.arguments[@"withZoom"];
  NSNumber *scrollBar = call.arguments[@"scrollBar"];
  NSNumber *withJavascript = call.arguments[@"withJavascript"];
  
  if (clearCache != (id)[NSNull null] && [clearCache boolValue]) {
    [[NSURLCache sharedURLCache] removeAllCachedResponses];
  }
  
  if (clearCookies != (id)[NSNull null] && [clearCookies boolValue]) {
    [[NSURLSession sharedSession] resetWithCompletionHandler:^{
    }];
  }
  
  if (userAgent != (id)[NSNull null]) {
    [[NSUserDefaults standardUserDefaults] registerDefaults:@{@"UserAgent": userAgent}];
  }
  
  WKWebViewConfiguration *configuration = [[WKWebViewConfiguration alloc] init];
  WKPreferences* preferences = [configuration preferences];
  if ([withJavascript boolValue]) {
    [preferences setJavaScriptEnabled:YES];
  } else {
    [preferences setJavaScriptEnabled:NO];
  }
  
  if (@available(iOS 11.0, *)) {
    WKWebsiteDataStore *store = WKWebsiteDataStore.nonPersistentDataStore;
    [authCookies enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
      NSHTTPCookie *cookie = [NSHTTPCookie
                              cookieWithProperties:[NSDictionary dictionaryWithObjectsAndKeys:
                                                    @".chai.finance", NSHTTPCookieDomain,
                                                    @"/", NSHTTPCookiePath,
                                                    @"TRUE", NSHTTPCookieSecure,
                                                    key, NSHTTPCookieName,
                                                    obj, NSHTTPCookieValue,
                                                    nil]];
      [store.httpCookieStore setCookie:cookie completionHandler:nil];
    }];
    [configuration setWebsiteDataStore:store];
  } else {
    WKUserContentController *userController = [[WKUserContentController alloc] init];
    [authCookies enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
      NSString *scriptStr = [NSString stringWithFormat:@"document.cookie='%@=%@; domain=.chai.finance; path=/; secure;';", key, obj];
      WKUserScript *script = [[WKUserScript alloc] initWithSource:scriptStr
                                                    injectionTime:WKUserScriptInjectionTimeAtDocumentStart
                                                 forMainFrameOnly:NO];
      [userController addUserScript:script];
    }];
    [configuration setUserContentController:userController];
  }
  
  self.webview = [[WKWebView alloc] initWithFrame:CGRectZero configuration:configuration];
  self.webview.allowsBackForwardNavigationGestures = YES;
  self.webview.UIDelegate = self;
  self.webview.navigationDelegate = self;
  self.webview.scrollView.delegate = self;
  self.webview.hidden = [hidden boolValue];
  self.webview.scrollView.showsHorizontalScrollIndicator = [scrollBar boolValue];
  self.webview.scrollView.showsVerticalScrollIndicator = [scrollBar boolValue];
  [self.webview addObserver:self forKeyPath:@"canGoBack" options:NSKeyValueObservingOptionNew|NSKeyValueObservingOptionOld|NSKeyValueObservingOptionInitial context:nil];
  
  _enableZoom = [withZoom boolValue];
  
  [self navigate:call];
}

-(void) initMethodHandler {
  NSLog(@"init method handler");
  __weak ChaiWebPlatformView *blockself = self;
  [channel setMethodCallHandler:^(FlutterMethodCall * _Nonnull call, FlutterResult _Nonnull result) {
    if ([@"eval" isEqualToString:call.method]) {
      [blockself evalJavascript:call completionHandler:^(NSString * response) {
        result(response);
      }];
    } else if ([@"resetAuthCookies" isEqualToString:call.method]) {
      [blockself resetAuthCookies:call];
      result(nil);
    } else if ([@"reloadUrl" isEqualToString:call.method]) {
      [blockself reloadUrl:call];
      result(nil);
    } else if ([@"show" isEqualToString:call.method]) {
      [blockself show];
      result(nil);
    } else if ([@"hide" isEqualToString:call.method]) {
      [blockself hide];
      result(nil);
    } else if ([@"stopLoading" isEqualToString:call.method]) {
      [blockself stopLoading];
      result(nil);
    } else if ([@"cleanCookies" isEqualToString:call.method]) {
      [blockself cleanCookies];
    } else if ([@"back" isEqualToString:call.method]) {
      [blockself back];
      result(nil);
    } else if ([@"forward" isEqualToString:call.method]) {
      [blockself forward];
      result(nil);
    } else if ([@"reload" isEqualToString:call.method]) {
      [blockself reload];
      result(nil);
    } else {
      result(FlutterMethodNotImplemented);
    }
    
  }];
}


- (void)cleanCookies {
  [[NSURLSession sharedSession] resetWithCompletionHandler:^{
  }];
}


- (void)evalJavascript:(FlutterMethodCall*)call
     completionHandler:(void (^_Nullable)(NSString * response))completionHandler {
  if (self.webview != nil) {
    NSString *code = call.arguments[@"code"];
    [self.webview evaluateJavaScript:code
                   completionHandler:^(id _Nullable response, NSError * _Nullable error) {
                     completionHandler([NSString stringWithFormat:@"%@", response]);
                   }];
  } else {
    completionHandler(nil);
  }
}



- (void)reloadUrl:(FlutterMethodCall*)call {
  if (self.webview != nil) {
    NSString *url = call.arguments[@"url"];
    NSURLRequest *request = [NSURLRequest requestWithURL:[NSURL URLWithString:url]];
    [self.webview loadRequest:request];
  }
}
- (void)stopLoading {
  if (self.webview != nil) {
    [self.webview stopLoading];
  }
}
- (void)back {
  if (self.webview != nil) {
    [self.webview goBack];
  }
}
- (void)forward {
  if (self.webview != nil) {
    [self.webview goForward];
  }
}
- (void)reload {
  if (self.webview != nil) {
    [self.webview reload];
  }
}


- (void)show {
  self.webview.hidden = false;
}

- (void)hide {
  self.webview.hidden = true;
}

- (void)resetAuthCookies:(FlutterMethodCall*)call {
  if (self.webview != nil) {
    NSDictionary *authCookies = call.arguments[@"authCookies"];
    WKWebViewConfiguration *configuration = self.webview.configuration;
    
    if (@available(iOS 11.0, *)) {
      WKWebsiteDataStore *store = WKWebsiteDataStore.nonPersistentDataStore;
      [authCookies enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
        NSHTTPCookie *cookie = [NSHTTPCookie
                                cookieWithProperties:[NSDictionary dictionaryWithObjectsAndKeys:
                                                      @".chai.finance", NSHTTPCookieDomain,
                                                      @"/", NSHTTPCookiePath,
                                                      @"TRUE", NSHTTPCookieSecure,
                                                      key, NSHTTPCookieName,
                                                      obj, NSHTTPCookieValue,
                                                      nil]];
        [store.httpCookieStore setCookie:cookie completionHandler:nil];
      }];
      [configuration setWebsiteDataStore:store];
    } else {
      WKUserContentController *userController = [[WKUserContentController alloc] init];
      [authCookies enumerateKeysAndObjectsUsingBlock:^(id  _Nonnull key, id  _Nonnull obj, BOOL * _Nonnull stop) {
        NSString *scriptStr = [NSString stringWithFormat:@"document.cookie='%@=%@; domain=.chai.finance; path=/; secure;';", key, obj];
        WKUserScript *script = [[WKUserScript alloc] initWithSource:scriptStr
                                                      injectionTime:WKUserScriptInjectionTimeAtDocumentStart
                                                   forMainFrameOnly:NO];
        [userController addUserScript:script];
      }];
      [configuration setUserContentController:userController];
    }
  }
}


- (void)navigate:(FlutterMethodCall*)call {
  NSString *url = call.arguments[@"url"];
  NSNumber *withLocalUrl = call.arguments[@"withLocalUrl"];
  if ( [withLocalUrl boolValue]) {
    NSURL *htmlUrl = [NSURL fileURLWithPath:url isDirectory:false];
    if (@available(iOS 9.0, *)) {
      [self.webview loadFileURL:htmlUrl allowingReadAccessToURL:htmlUrl];
    } else {
      @throw @"not available on version earlier than ios 9.0";
    }
  } else {
    NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:url]];
    //              [request setHTTPShouldHandleCookies:YES];
    NSDictionary *headers = call.arguments[@"headers"];
    
    if (headers != nil) {
      //                    [request setAllHTTPHeaderFields:headers];
    }
    //
    NSLog(@"%@", request.allHTTPHeaderFields.description);
    [self.webview loadRequest:request];
    //              NSLog(@"%@ %@", [[NSHTTPCookieStorage sharedHTTPCookieStorage] cookies], [[NSHTTPCookieStorage sharedHTTPCookieStorage] cookiesForURL:[NSURL URLWithString:url]]);
  }
}

- (UIView *)view {
  return self.webview;
}

- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
  if (object == self.webview && [keyPath isEqualToString:@"canGoBack"]) {
    BOOL canGoBack = [change[@"new"] boolValue];
    [channel invokeMethod:@"onCanGoBackChanged" arguments:@{@"canGoBack": [NSNumber numberWithBool:canGoBack]}];
  }
}

- (bool)checkInvalidUrl:(NSURL*)url {
  NSString* urlString = url != nil ? [url absoluteString] : nil;
  if (_invalidUrlRegex != [NSNull null] && urlString != nil) {
    NSError* error = NULL;
    NSRegularExpression* regex =
    [NSRegularExpression regularExpressionWithPattern:_invalidUrlRegex
                                              options:NSRegularExpressionCaseInsensitive
                                                error:&error];
    NSTextCheckingResult* match = [regex firstMatchInString:urlString
                                                    options:0
                                                      range:NSMakeRange(0, [urlString length])];
    return match != nil;
  } else {
    return false;
  }
}

#pragma mark -- WkWebView Delegate
- (void)webView:(WKWebView *)webView decidePolicyForNavigationAction:(WKNavigationAction *)navigationAction
decisionHandler:(void (^)(WKNavigationActionPolicy))decisionHandler {
  
  BOOL isInvalid = [self checkInvalidUrl: navigationAction.request.URL];
  
  id data = @{@"url": navigationAction.request.URL.absoluteString,
              @"type": isInvalid ? @"abortLoad" : @"shouldStart",
              @"navigationType": [NSNumber numberWithInt:navigationAction.navigationType]};
  [channel invokeMethod:@"onState" arguments:data];
  
  if (navigationAction.navigationType == WKNavigationTypeBackForward) {
    [channel invokeMethod:@"onBackPressed" arguments:nil];
  } else if (!isInvalid) {
    id data = @{@"url": navigationAction.request.URL.absoluteString};
    [channel invokeMethod:@"onUrlChanged" arguments:data];
  }
  
  if (_enableAppScheme ||
      ([webView.URL.scheme isEqualToString:@"http"] ||
       [webView.URL.scheme isEqualToString:@"https"] ||
       [webView.URL.scheme isEqualToString:@"about"])) {
        if (isInvalid) {
          decisionHandler(WKNavigationActionPolicyCancel);
        } else {
          decisionHandler(WKNavigationActionPolicyAllow);
        }
      } else {
        decisionHandler(WKNavigationActionPolicyCancel);
      }
}

- (WKWebView *)webView:(WKWebView *)webView createWebViewWithConfiguration:(WKWebViewConfiguration *)configuration
   forNavigationAction:(WKNavigationAction *)navigationAction windowFeatures:(WKWindowFeatures *)windowFeatures {
  
  if (!navigationAction.targetFrame.isMainFrame) {
    [webView loadRequest:navigationAction.request];
  }
  
  return nil;
}

- (void)webView:(WKWebView *)webView didStartProvisionalNavigation:(WKNavigation *)navigation {
  [channel invokeMethod:@"onState" arguments:@{@"type": @"startLoad", @"url": webView.URL.absoluteString}];
}

- (void)webView:(WKWebView *)webView didFinishNavigation:(WKNavigation *)navigation {
  [channel invokeMethod:@"onState" arguments:@{@"type": @"finishLoad", @"url": webView.URL.absoluteString}];
}

- (void)webView:(WKWebView *)webView didFailNavigation:(WKNavigation *)navigation withError:(NSError *)error {
  [channel invokeMethod:@"onError" arguments:@{@"code": [NSString stringWithFormat:@"%ld", error.code], @"error": error.localizedDescription}];
}

- (void)webView:(WKWebView *)webView decidePolicyForNavigationResponse:(WKNavigationResponse *)navigationResponse decisionHandler:(void (^)(WKNavigationResponsePolicy))decisionHandler {
  if ([navigationResponse.response isKindOfClass:[NSHTTPURLResponse class]]) {
    NSHTTPURLResponse * response = (NSHTTPURLResponse *)navigationResponse.response;
    
    [channel invokeMethod:@"onHttpError" arguments:@{@"code": [NSString stringWithFormat:@"%ld", response.statusCode], @"url": webView.URL.absoluteString}];
  }
  decisionHandler(WKNavigationResponsePolicyAllow);
}

#pragma mark -- UIScrollViewDelegate
- (void)scrollViewWillBeginDragging:(UIScrollView *)scrollView {
  if (scrollView.pinchGestureRecognizer.isEnabled != _enableZoom) {
    scrollView.pinchGestureRecognizer.enabled = _enableZoom;
  }
}

- (void) scrollViewDidScroll:(UIScrollView *)scrollView {
  id xDirection = @{@"xDirection": @(scrollView.contentOffset.x) };
  [channel invokeMethod:@"onScrollXChanged" arguments:xDirection];
  
  id yDirection = @{@"yDirection": @(scrollView.contentOffset.y) };
  [channel invokeMethod:@"onScrollYChanged" arguments:yDirection];
}


- (void)webView:(WKWebView *)webView runJavaScriptConfirmPanelWithMessage:(NSString *)message initiatedByFrame:(WKFrameInfo *)frame completionHandler:(void (^)(BOOL))completionHandler {
  UIAlertController* alertController = [UIAlertController alertControllerWithTitle:message message:@"" preferredStyle:UIAlertControllerStyleAlert];
  
  [alertController addAction:[UIAlertAction actionWithTitle:@"확인" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
    completionHandler(YES);
  }]];
  
  [alertController addAction:[UIAlertAction actionWithTitle:@"취소" style:UIAlertActionStyleDefault handler:^(UIAlertAction * _Nonnull action) {
    completionHandler(NO);
  }]];
  
  [viewController presentViewController:alertController animated:YES completion:nil];
}

- (BOOL)gestureRecognizerShouldBegin:(UIGestureRecognizer *)gestureRecognizer {
  if (self.webview != nil) {
    return !self.webview.canGoBack;
  }
  return NO;
}

@end
