import 'dart:async';

import 'package:flutter/foundation.dart';
import 'package:flutter/material.dart';
import 'package:flutter/rendering.dart';
import 'package:flutter/services.dart';

import 'webview_controller.dart';
import 'base.dart';

class WebviewScaffold extends StatefulWidget {
  const WebviewScaffold(
      {Key key,
        this.appBar,
        @required this.url,
        this.headers,
        this.withJavascript,
        this.clearCache,
        this.clearCookies,
        this.enableAppScheme,
        this.userAgent,
        this.primary = true,
        this.persistentFooterButtons,
        this.bottomNavigationBar,
        this.withZoom,
        this.withLocalStorage,
        this.withLocalUrl,
        this.scrollBar,
        this.supportMultipleWindows,
        this.appCacheEnabled,
        this.hidden = false,
        this.initialChild,
        this.allowFileURLs,
        this.resizeToAvoidBottomInset = false,
        this.invalidUrlRegex,
        this.geolocationEnabled,
        this.authCookies,
        this.urlCallback})
      : super(key: key);

  final PreferredSizeWidget appBar;
  final String url;
  final Map<String, String> headers;
  final bool withJavascript;
  final bool clearCache;
  final bool clearCookies;
  final bool enableAppScheme;
  final String userAgent;
  final bool primary;
  final List<Widget> persistentFooterButtons;
  final Widget bottomNavigationBar;
  final bool withZoom;
  final bool withLocalStorage;
  final bool withLocalUrl;
  final bool scrollBar;
  final bool supportMultipleWindows;
  final bool appCacheEnabled;
  final bool hidden;
  final Widget initialChild;
  final bool allowFileURLs;
  final bool resizeToAvoidBottomInset;
  final String invalidUrlRegex;
  final bool geolocationEnabled;
  final Map authCookies;
  final Function(String) urlCallback;

  Map<String, dynamic> getArgs() {
    final args = <String, dynamic>{
      'url': url,
      'withJavascript': withJavascript ?? true,
      'clearCache': clearCache ?? false,
      'hidden': hidden ?? false,
      'clearCookies': clearCookies ?? false,
      'enableAppScheme': enableAppScheme ?? true,
      'userAgent': userAgent,
      'withZoom': withZoom ?? false,
      'withLocalStorage': withLocalStorage ?? true,
      'withLocalUrl': withLocalUrl ?? false,
      'scrollBar': scrollBar ?? true,
      'supportMultipleWindows': supportMultipleWindows ?? false,
      'appCacheEnabled': appCacheEnabled ?? false,
      'allowFileURLs': allowFileURLs ?? false,
      // 'useWideViewPort': useWideViewPort ?? false,
      'invalidUrlRegex': invalidUrlRegex,
      'geolocationEnabled': geolocationEnabled ?? false,
      'authCookies': authCookies ?? null,
    };
    return args;
  }

  @override
  _WebviewScaffoldState createState() => _WebviewScaffoldState();
}

class _WebviewScaffoldState extends State<WebviewScaffold> {
  StreamSubscription<WebViewStateChanged> _onStateChanged;
  StreamSubscription<bool> _onCanGoBackChanged;
  bool loaded = false;
  bool canGoBack = false;
  var controller;

  initController(id) {
    controller = WebViewController(id);

    _onStateChanged =
        controller.onStateChanged.listen((WebViewStateChanged state) {
          if (!loaded && state.type == WebViewState.finishLoad) {
            setState(() {
              loaded = true;
            });
            controller.show();
          }
          if(widget.urlCallback != null) {
            widget.urlCallback(state.url);
          }
        });

    _onCanGoBackChanged = controller.onCanGoBackChanged.listen((v) {
      if (mounted) {
        setState(() {
          canGoBack = v;
        });
      } else {
        canGoBack = v;
      }
    });
  }

  @override
  dispose() {
    print('webview controller disposing!!!!!!!!!!!!!!!!!!!!!');
    _onCanGoBackChanged?.cancel();
    _onStateChanged?.cancel();
    controller?.dispose();
    super.dispose();
  }

  @override
  Widget build(BuildContext context) {
    return Scaffold(
      appBar: widget.appBar,
      resizeToAvoidBottomInset: widget.resizeToAvoidBottomInset,
      persistentFooterButtons: widget.persistentFooterButtons,
      bottomNavigationBar: widget.bottomNavigationBar,
      body: WillPopScope(
        onWillPop: canGoBack ? () => Future.value(false) : null,
        child: SafeArea(
          top: true,
          bottom: false,
          child: Stack(
            children: <Widget>[
              getPlatformView(defaultTargetPlatform),
              loaded ? Container() : Center(child: CircularProgressIndicator()),
            ],
          ),
        ),
      ),
    );
  }

  Widget getPlatformView(TargetPlatform targetPlatform){
    if(targetPlatform == TargetPlatform.android){
      return AndroidView(
        viewType: 'chai_webview',
        creationParams: widget.getArgs(),
        creationParamsCodec: const StandardMessageCodec(),
        onPlatformViewCreated: initController,
      );
    }else if(targetPlatform == TargetPlatform.iOS){
      return UiKitView(
        viewType: 'chai_webview',
        creationParams: widget.getArgs(),
        creationParamsCodec: const StandardMessageCodec(),
        onPlatformViewCreated: initController,
      );
    }else{
      return Text('$defaultTargetPlatform is not supported');
    }
  }
}
